import { Component, OnInit } from '@angular/core';
import { DataService } from './data.service';
import { ClientCollection, UnidadOrganizativa } from './model';
import { MatDialog } from '@angular/material/dialog';
import { CreateOrganizationalUnitComponent } from './organizational-units/create-organizational-unit/create-organizational-unit.component';
import { DeleteModalComponent } from './delete-modal/delete-modal.component';
import { CreateClientComponent } from './clients/create-client/create-client.component';
import { EditOrganizationalUnitComponent } from './organizational-units/edit-organizational-unit/edit-organizational-unit.component';
import { EditClientComponent } from './clients/edit-client/edit-client.component';
import { DeleteGroupsModalComponent } from "./delete-groups-modal/delete-groups-modal.component";
import { ShowOrganizationalUnitComponent} from "./organizational-units/show-organizational-unit/show-organizational-unit.component";
import {ToastrService} from "ngx-toastr";
import {TreeViewComponent} from "./tree-view/tree-view.component";
import {MatBottomSheet} from "@angular/material/bottom-sheet";
import {LegendComponent} from "./legend/legend.component";

@Component({
  selector: 'app-groups',
  templateUrl: './groups.component.html',
  styleUrls: ['./groups.component.css']
})
export class GroupsComponent implements OnInit {
  organizationalUnits: UnidadOrganizativa[] = [];
  selectedUnidad: UnidadOrganizativa | null = null;
  selectedDetail: any | null = null;  // Nueva variable para el detalle del elemento seleccionado
  children: any[] = [];
  breadcrumb: string[] = [];
  clientsData: any[] = [];  // Nueva variable para almacenar los datos de clients
  breadcrumbData: any[] = [];  // Almacenar datos de breadcrumb para navegar
  loading:boolean = false;
  loadingChildren:boolean = false;
  searchTerm: string = '';
constructor(
  private dataService: DataService,
  public dialog: MatDialog,
  private toastService: ToastrService,
  private _bottomSheet: MatBottomSheet
  ) {}

  ngOnInit(): void {
    this.search();
  }

  search(): void {
    this.loading = true;
    this.dataService.getOrganizationalUnits(this.searchTerm).subscribe(
      data => {
        this.organizationalUnits = data;
        this.loading = false; // Desactivar el spinner después de obtener los datos
      },
      error => {
        console.error('Error fetching unidades organizativas', error);
        this.loading = false; // Desactivar el spinner en caso de error
      }
    );
  }


  onSelectUnidad(unidad: UnidadOrganizativa): void {
    this.selectedUnidad = unidad;
    this.selectedDetail = unidad;  // Mostrar detalles de la unidad seleccionada
    this.breadcrumb = [unidad.name];
    this.breadcrumbData = [unidad];
    this.loadChildrenAndClients(unidad.id);
  }

  onSelectChild(child: any): void {
    this.selectedDetail = child;  // Mostrar detalles del niño seleccionado
    if (child.type !== 'client' && child.uuid && child.id) {
      this.breadcrumb.push(child.name || child.name);
      this.breadcrumbData.push(child);
      this.loadChildrenAndClients(child.id);
    }
  }

  navigateToBreadcrumb(index: number): void {
    this.breadcrumb = this.breadcrumb.slice(0, index + 1);
    const target = this.breadcrumbData[index];
    this.breadcrumbData = this.breadcrumbData.slice(0, index + 1);
    if (target.type === 'client') {
      this.selectedDetail = target;
    } else {
      this.loadChildrenAndClients(target.id);
    }
  }

  loadChildrenAndClients(id: string): void {
    this.loadingChildren = true
    this.dataService.getChildren(id).subscribe(
      childrenData => {
        console.log('Children data:', childrenData);
        this.dataService.getClients(id).subscribe(
          clientsData => {
            this.clientsData = clientsData;  // Almacenar clientsData para pasarlo al componente hijo
            const newChildren = [...childrenData, ...clientsData];

            if (newChildren.length > 0) {
              this.children = newChildren;
            } else {
              this.children = []; // Limpiar card2 cuando no hay elementos
            }
            this.loadingChildren = false
          },
          error => {
            console.error('Error fetching clients', error);
            this.clientsData = [];  // Limpiar clientsData en caso de error
            this.children = []; // Limpiar card2 en caso de error
            this.loadingChildren = false
          }
        );
      },
      error => {
        console.error('Error fetching children', error);
        this.children = []; // Limpiar card2 en caso de error
        this.loadingChildren = false
      }
    );
  }

  addOU(event: MouseEvent, parent:any = null): void {
    event.stopPropagation();
    const dialogRef = this.dialog.open(CreateOrganizationalUnitComponent, { data: { parent }, width: '700px'});
    dialogRef.afterClosed().subscribe(() => {
      this.dataService.getOrganizationalUnits().subscribe(
        data => {
          this.organizationalUnits = data
          this.loadChildrenAndClients(parent.id);
        },
        error => console.error('Error fetching unidades organizativas', error)
      );
    });
  }

  addClient(event: MouseEvent, organizationalUnit:any = null): void {
    event.stopPropagation();

    const dialogRef = this.dialog.open(CreateClientComponent, { data: { organizationalUnit }, width: '700px'});

    // Subscribirse al evento unitAdded del componente de creación después de cerrar el diálogo
    dialogRef.afterClosed().subscribe(() => {
      this.dataService.getOrganizationalUnits().subscribe(
        data => {
          this.organizationalUnits = data
          this.loadChildrenAndClients(organizationalUnit.id);
        },
        error => console.error('Error fetching unidades organizativas', error)
      );
    });
  }
  onDeleteClick(event: MouseEvent, uuid: string, name: string, type: string): void {
    event.stopPropagation();
    if (type === 'client') {
      const dialogRef = this.dialog.open(DeleteModalComponent, {
        width: '400px',
        data: { name }
      });

      dialogRef.afterClosed().subscribe(result => {
        if (result) {
          this.dataService.deleteElement(uuid, type).subscribe(
            () => {
              this.loadChildrenAndClients(this.selectedUnidad?.id || '');
              this.dataService.getOrganizationalUnits().subscribe(
                data => this.organizationalUnits = data,
                error => console.error('Error fetching unidades organizativas', error)
              );
              this.openSnackBar(false, 'Entidad eliminada exitosamente')
            },
            error =>  {
              console.error('Error deleting element', error)
              this.openSnackBar(true, error.error['hydra:description'])
            }
          );
        }
      });
    } else {
      const dialogDeleteGroupRef = this.dialog.open(DeleteGroupsModalComponent, {
        width: '400px',
        data: { name }
      });

      dialogDeleteGroupRef.afterClosed().subscribe(result => {
        if (result && result === 'delete') {
          this.dataService.deleteElement(uuid, type).subscribe(
            () => {
              this.loadChildrenAndClients(this.selectedUnidad?.id || '');
              this.dataService.getOrganizationalUnits().subscribe(
                data => this.organizationalUnits = data,
                error => console.error('Error fetching unidades organizativas', error)
              );
              this.openSnackBar(false, 'Entidad eliminada exitosamente')
            },
            error => {
              console.error('Error deleting element', error)
              this.openSnackBar(true, error.error['hydra:description'])
            }
          );
        } else if (result && result === 'change') {
          this.dataService.changeParent(uuid).subscribe(
            () => {
              this.loadChildrenAndClients(this.selectedUnidad?.id || '');
              this.dataService.getOrganizationalUnits().subscribe(
                data => this.organizationalUnits = data,
                error => console.error('Error fetching unidades organizativas', error)
              );
              this.openSnackBar(false, 'Entidad eliminada exitosamente')
            },
            error => {
              console.error('Error deleting element', error)
              this.openSnackBar(true, error.error['hydra:description'])
            }
          );
        }
      });
    }
  }

  onEditClick(event: MouseEvent, type: any, uuid: string): void {
    event.stopPropagation();
    console.log('Tipo del elemento a editar:', type);
    console.log('UUID del elemento a editar:', uuid);
    if (type != "client") {
      const dialogRef = this.dialog.open(EditOrganizationalUnitComponent, { data: { uuid }, width: '700px'});
    } else {
      console.log('Editar cliente');
      const dialogRef = this.dialog.open(EditClientComponent, { data: { uuid }, width: '700px' } );
    }
  }

  onShowClick(event: MouseEvent, data: any): void {
    event.stopPropagation();
    if (data.type != "client") {
      const dialogRef = this.dialog.open(ShowOrganizationalUnitComponent, { data: { data }, width: '700px'});
    }
  }

  onTreeClick(event: MouseEvent, data: any): void {
    event.stopPropagation();
    if (data.type != "client") {
      const dialogRef = this.dialog.open(TreeViewComponent, { data: { data }, width: '800px'});
    }
  }

  openSnackBar(isError: boolean, message: string) {
    if (isError) {
      this.toastService.error(' Error al eliminar la entidad: ' + message, 'Error');
    } else
      this.toastService.success(message, 'Éxito');
  }

  openBottomSheet(): void {
    this._bottomSheet.open(LegendComponent);
  }
}
